const _ = require('lodash');
const Mustache = require('mustache');

module.exports = class DialogParsing {
  constructor() {}

  static resolveDialog(session, outerItem) {
    Mustache.templateCache = undefined;

    const item = _.chain(outerItem)
      .omit('children')
      .cloneDeep()
      .value();

    const itemString = JSON.stringify(item);
    if (!itemString.includes('{{')) {
      return outerItem;
    }

    session = _.cloneDeep(session);

    let result = item;
    if (item) {
      if (
        item.iterator &&
        item.iterator.enabled &&
        item.iterator.itemName &&
        item.iterator.variableName
      ) {
        result = DialogParsing.resolveIterator(session, item);
      } else {
        result = DialogParsing.replaceVariablesDialog(session, item);
      }
    }

    // Clean up remaining mustache variables
    result = JSON.parse(Mustache.render(JSON.stringify(result), {}));

    result.children = outerItem.children;
    return result;
  }

  static replaceVariablesDialog(session, item) {
    let jsonString = null;

    if (item) {
      jsonString = JSON.stringify(item);
      jsonString = Mustache.render(jsonString, session);
    }
    try {
      if (!jsonString) {
        throw new Error('Empty jsonString');
      }
      return JSON.parse(jsonString);
    } catch (e) {
      console.log(e);
      return item;
    }
  }

  static resolveIterator(session, item) {
    try {
      const itemClone = _.cloneDeep(item);
      if (itemClone.value && itemClone.value.length === 1) {
        const iterateVariable = _.get(session, itemClone.iterator.variableName);
        const itemVariable = itemClone.iterator.itemName;
        if (iterateVariable && itemVariable) {
          const template = itemClone.value[0];
          itemClone.value = [];

          if (iterateVariable.length) {
            for (const newContext of iterateVariable) {
              const one = _.cloneDeep(template);
              session[itemVariable] = newContext;
              const newOne = DialogParsing.replaceVariablesDialog(session, one);
              itemClone.value.push(newOne);
            }
          } else {
            throw new Error('Returned array has empty data');
          }
        } else {
          throw new Error("Input variables don't exist");
        }

        return itemClone;
      }
      throw new Error("Item doesn't have value property or value is not unique");
    } catch (e) {
      console.log(e);
      item.value = []; // Prevent showing result
      if (item.iterator && item.iterator.onEmptyGoToDialog) {
        item.goToDialog = item.iterator.onEmptyGoToDialog;
      }
      return item;
    }
  }
};
